/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdccompositejob.h"

#include <QListIterator>

KDCCompositeJob::KDCCompositeJob( KDCCompositeJob::AutoConnect mode, QObject *parent )
	: KDCJob( parent )
{
	p_count = 0;
	p_mode = mode;
}

KDCCompositeJob::~KDCCompositeJob()
{
	while( !p_jobList.isEmpty() )
		removeSubjob( p_jobList.last() );
}

void KDCCompositeJob::addSubjob( KDCJob *job )
{
	p_jobList.append( job );
	if ( p_mode == KDCCompositeJob::Yes )
	{
		connect( job, SIGNAL( speed( double ) ), this, SIGNAL( speed( double ) ) );
		connect( job, SIGNAL( progress( int ) ), this, SIGNAL( progress( int ) ) );
		connect( job, SIGNAL( percent( int ) ), this, SIGNAL( percent( int ) ) );
		connect( job, SIGNAL( jobTotalSize( int ) ), this, SIGNAL( jobTotalSize( int ) ) );
		connect( job, SIGNAL( infoMessage( const QString& ) ), this, SIGNAL( infoMessage( const QString& ) ) );
		connect( job, SIGNAL( errorMessage( const QString& ) ), this, SIGNAL( errorMessage( const QString& ) ) );
		connect( job, SIGNAL( jobMainProgram( const QString& ) ), this, SIGNAL( jobMainProgram( const QString& ) ) );
	}
	p_totalSize += job->totalSize();
	if ( job->totalSize() > 0 )
		p_count += job->count();
}

void KDCCompositeJob::insertSubjob( KDCJob *job, int index )
{
	p_jobList.insert( index, job );
	if ( p_mode = KDCCompositeJob::Yes )
	{
		connect( job, SIGNAL( speed( double ) ), this, SIGNAL( speed( double ) ) );
		connect( job, SIGNAL( progress( int ) ), this, SIGNAL( progress( int ) ) );
		connect( job, SIGNAL( percent( int ) ), this, SIGNAL( percent( int ) ) );
		connect( job, SIGNAL( jobTotalSize( int ) ), this, SIGNAL( jobTotalSize( int ) ) );
		connect( job, SIGNAL( infoMessage( const QString& ) ), this, SIGNAL( infoMessage( const QString& ) ) );
		connect( job, SIGNAL( errorMessage( const QString& ) ), this, SIGNAL( errorMessage( const QString& ) ) );
		connect( job, SIGNAL( jobMainProgram( const QString& ) ), this, SIGNAL( jobMainProgram( const QString& ) ) );
	}
	p_totalSize += job->totalSize();
	if ( job->totalSize() > 0 )
		p_count += job->count();
}

int KDCCompositeJob::subjobCount() const
{
	return p_jobList.count();
}

int KDCCompositeJob::count() const
{
	return p_count;
}

bool KDCCompositeJob::hasSubjobs()
{
	if ( p_jobList.size() > 0 )
		return TRUE;
	else
		return FALSE;
}

void KDCCompositeJob::removeSubjob( KDCJob *job )
{
	p_jobList.removeAll( job );
	if ( job )
	{
		job->disconnect();
		job->deleteLater();
	}
}

KDCJob* KDCCompositeJob::firstSubjob() const
{
	return p_jobList.first();
}

KDCJob* KDCCompositeJob::lastSubjob() const
{
	return p_jobList.last();
}
