/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdccpuinfojob.h"

#include <QByteArray>

KDCCpuInfoJob::KDCCpuInfoJob( QObject *parent )
	: KDCJob( parent )
{
	p_process = new KProcess( this );
	p_process->setOutputChannelMode( KProcess::SeparateChannels );
	connect( p_process, SIGNAL( readyReadStandardOutput() ), this, SLOT( readOutput() ) );
	QStringList arg;
	arg << "-d" << "4" << "-b";
	p_process->setProgram( "top", arg );
	p_program = QString();
}

KDCCpuInfoJob::~KDCCpuInfoJob()
{
	qDebug( "~cpuJob" );
	if ( p_process )
	{
		p_process->disconnect();
		if ( p_process->state() == QProcess::Running )
			p_process->close();
		delete p_process;
	}
}

void KDCCpuInfoJob::start()
{
	p_process->start();
}

void KDCCpuInfoJob::readOutput()
{
	QByteArray buffer( p_process->readAllStandardOutput() );
	int idx = buffer.indexOf( p_program );
	if ( idx != -1 )
	{
		QByteArray temp = buffer.mid( idx - 20, 4 ).trimmed();
		emit info( qRound( temp.toDouble()*10 ) );
	}
}

void KDCCpuInfoJob::setProgram( const QString& program )
{
	if ( p_program != program )
	{
		p_program = program;
		if ( p_process->state() == QProcess::Running )
		{
			p_process->close();
			p_process->start();
		}
	}
}
