/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcgrabframejob.h"

#include <QStringList>

#include <KLocale>

KDCGrabFrameJob::KDCGrabFrameJob( int frame, const QString& movieFile, const QString& outDir, QObject *parent )
	: KDCJob( parent )
{
	p_process = new KProcess( this );
	p_process->setOutputChannelMode( KProcess::MergedChannels );
	connect( p_process, SIGNAL( finished( int , QProcess::ExitStatus ) ), this, SLOT( on_finished( int , QProcess::ExitStatus ) ) );
	connect( p_process, SIGNAL( readyReadStandardOutput() ), this, SLOT( on_readyReadStandardOutput() ) );

	QStringList arg;
	arg << "-vo" << "jpeg:outdir=" + outDir << "-ss" << QString::number( frame );
	arg << "-nosound" << "-noautosub" << "-frames" << "1" << movieFile;

	p_process->setProgram( "mplayer", arg );
	setTotalSize( 100 );
}

KDCGrabFrameJob::~KDCGrabFrameJob()
{
	if ( p_process )
	{
		if ( p_process->state() == QProcess::Running )
		{
			disconnect( p_process, 0, 0, 0 );
			p_process->close();
		}
		delete p_process;
	}
}

void KDCGrabFrameJob::on_readyReadStandardOutput()
{
	//TODO
}

void KDCGrabFrameJob::start()
{
	emit percent( 0 );
	emit infoMessage( i18n("Grabing frame...") );
	p_process->start();
}

void KDCGrabFrameJob::on_finished( int code , QProcess::ExitStatus status )
{
	qDebug( "KDCGrabFrameJob::finished()::Codec = %d, status = %d", code, status );
	if ( status == QProcess::CrashExit && !flag( 1 ) )
		setError( i18n("There was an error during process. Error code is %1").arg( code ) );
	else
		emit percent( 100 );
	emit finished( this );
}
