/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcmoviegroupprocesscompositejob.h"

#include "../kdcsubtitlestreamgroup.h"
#include "../kdcdisc.h"
#include "../kdcfilefactory.h"

#include "kdcsubtitleconvertjob.h"
#include "kdcmovieconvertjob.h"
#include "kdcapplysubtitlejob.h"
#include "kdcsubtitlecatjob.h"
#include "kdcsubtitlecreatexmlfilejob.h"

#include <KLocale>

KDCMovieGroupProcessCompositeJob::KDCMovieGroupProcessCompositeJob( const KDCMovieGroup *movieGroup , QObject *parent )
	: KDCCompositeJob( KDCCompositeJob::Yes, parent )
{
	p_file = movieGroup->outputName();

	//first we must create movie
	KDCMovieConvertJob *eJob = new KDCMovieConvertJob( movieGroup );
	connect( eJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
	addSubjob( eJob );

	//next we convert subtitles
	KDCMovie *movie = movieGroup->movie( 0 );
	if ( movie->subtitleStreamGroup()->count() > 0 )
	{
		KDCApplySubtitleJob *sJob;
		KDCSubtitleConvertJob *cJob;
		KDCSubtitleCreateXMLFileJob *xJob;
		KDCSubtitleCatJob *aJob;

		int j;
		int totalLength;
		KDCSubtitleStreamGroup *group = movie->subtitleStreamGroup();
		KDCMovie *tmp;
		// first we must convert subtitle to correct fps
		for ( int i=0; i<group->count(); ++i )
		{
			for ( j=0; j<movieGroup->count(); ++j )
			{
				movie = movieGroup->movie( j );
				group = movie->subtitleStreamGroup();

				if ( j == 0 )
					totalLength = 0;
				else
				{
					tmp = movieGroup->movie( j - 1 );
					totalLength += tmp->videoStream()->metaData()->frameCount();
				}

				cJob = new KDCSubtitleConvertJob( i, movie, totalLength, this );
				connect( cJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
				addSubjob( cJob );
			}

			// next we must join all subtitle streams
			aJob = new KDCSubtitleCatJob( movieGroup, i, this );
			connect( aJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
			addSubjob( aJob );

			// next we must create xml file for spumux
			xJob = new KDCSubtitleCreateXMLFileJob( i, movieGroup, this );
			connect( xJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
			addSubjob( xJob );

			sJob = new KDCApplySubtitleJob( i, movieGroup, this );
			connect( sJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
			addSubjob( sJob );
		}
	}
}

KDCMovieGroupProcessCompositeJob::~KDCMovieGroupProcessCompositeJob()
{

}

void KDCMovieGroupProcessCompositeJob::start()
{
	emit infoMessage( i18n("Start processing movie group ") + p_file );
	firstSubjob()->start();
}


void KDCMovieGroupProcessCompositeJob::on_jobFinished( KDCJob *job )
{
	if ( job->hasErrors() )
	{
		if ( job->error() != "0" )
			emit errorMessage( job->error() );
		removeSubjob( job );
		setError( "0" );
		emit finished( this );
	}
	else
	{
		removeSubjob( job );
		if ( hasSubjobs() )
			firstSubjob()->start();
		else
		{
			emit infoMessage( i18n("Processing movie group ") + p_file + i18n(" sucesfully finished") );
			emit finished( this );
		}
	}
}
