/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdconflypreviewjob.h"

#include "../kdcconfig.h"
#include "../kdcmoviegroup.h"
#include "../kdcfilefactory.h"
#include "../widgets/kdcmplayerwidget.h"

#include <QProgressBar>
#include <QHBoxLayout>

#include <KLocale>
#include <KMessageBox>

KDCOnFlyPreviewJob::KDCOnFlyPreviewJob( QWidget *dialogParent, QObject *parent )
	: KDCCompositeJob( KDCCompositeJob::No, parent )
{
	p_movie = NULL;
	p_parent = dialogParent;
}

KDCOnFlyPreviewJob::~KDCOnFlyPreviewJob()
{
	setFlag( 3, FALSE );
	qDebug( "~KDCOnFlyPreviewJob::begin" );
	if ( p_dlg )
	{
		p_dlg->disconnect();
		delete p_dlg;
	}
	emit done();
	qDebug( "~KDCOnFlyPreviewJob::end" );
}

void KDCOnFlyPreviewJob::start()
{

}

void KDCOnFlyPreviewJob::show()
{
	qDebug( "KDCOnFlyPreviewJob::show()::begin" );
	if ( !p_movie )
		return;

	if ( p_mJob && p_mJob->isRunning() )
		return;

	setFlag( 1, FALSE );
	setFlag( 2, FALSE );
	setFlag( 3, TRUE );

	p_dlg = new KDCDialog( p_parent, Qt::WindowTitleHint );
	p_dlg->setWindowTitle( i18n("KDVDCreator - preview") );
	connect( p_dlg, SIGNAL( finished() ), this, SLOT( close() ) );

	KDCMPlayerWidget *p_mpl = new KDCMPlayerWidget;
	p_mpl->showControlButton( KDCMPlayerWidget::STOP, FALSE );
	p_mpl->showTimeSlider( FALSE );
	p_mpl->setWindowSize( 433, 244 );
	double aspect = 4.0/3.0;
	if ( p_movie->videoStream()->aspectRatio() == KDC::WideScreen )
		aspect = 16.0/9.0;
	p_mpl->setMovieProperties( p_movie->videoStream()->width(), p_movie->videoStream()->height(), aspect );

	QWidget *widget = p_mpl->controlPanel();
	QHBoxLayout *layout = static_cast<QHBoxLayout*>( widget->layout() );
	p_bar = new QProgressBar;
	p_bar->setMaximumHeight( 20 );
	p_bar->setFormat( i18n("Buffer level %p%") );
	layout->insertWidget( 3, p_bar );

	p_dlg->setCloseEnabled( TRUE );
	p_dlg->setMainWidget( p_mpl );

	p_mJob = new KDCMPlayerJob( this );
	addSubjob( p_mJob );
	p_mJob->setFile( p_fileName );
	p_mJob->setWindowID( p_mpl->windowID() );
	p_mJob->setVideoDriver( KDCConfig::driver() );

	connect( p_mJob, SIGNAL( progress( int ) ), p_mpl, SLOT( setTimeSeek( int ) ) );
	connect( p_mJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_mplayerJobFinished( KDCJob * ) ) );
	connect( p_mpl, SIGNAL( pauseClicked() ), p_mJob, SLOT( forcePause() ) );
	connect( p_mpl, SIGNAL( playClicked() ), this, SLOT( startClicked() ) );

	p_dlg->show();

	qDebug( "KDCOnFlyPreviewJob::slotStart()::begin" );
}

void KDCOnFlyPreviewJob::setCurrentFrame( int value )
{
	if ( flag( 3 ) )
	{
		p_pos = qRound( value/p_movie->fps() );
		if ( value > 150 )
			setFlag( 1, TRUE );
		else
			p_bar->setValue( qBound( 0, qRound( 100 - 14.287*( 7 - p_pos ) ), 100 ) );

		if ( flag( 1 ) )
		{
			if ( flag( 2 ) )
			{
				int currPos = p_mJob->currentTime() + 5;
				p_bar->setValue( qBound( 0, (p_pos - currPos + 5)*20, 100 ) );
				if ( currPos > p_pos )
					p_mJob->pause();
				else
					p_mJob->unpause();
			}
			else
			{
				setFlag( 2, TRUE );
				startClicked();
			}
		}
	}
}

void KDCOnFlyPreviewJob::startClicked()
{
	if ( !p_mJob )
		return;
	if ( p_mJob->isRunning() )
		return;

	p_mJob->startAndSeek( qMax( 0, p_pos - 5 ) );
}

void KDCOnFlyPreviewJob::setCurrentMovie( QTreeWidgetItem *movie )
{
	qDebug( "KDCOnFlyPreviewJob::setCurrentMovie()" );
	if ( movie )
	{
		if ( movie->type() == KDC::Movie )
		{
			p_movie = static_cast<KDCMovie*>( movie );
			p_fileName = KDCFileFactory::movieName( p_movie );
		}
		else
		{
			KDCMovieGroup *group = static_cast<KDCMovieGroup*>( movie );
			p_movie = group->movie( 0 );
			p_fileName = KDCFileFactory::movieName( group );
		}
	}
	else
		p_movie = NULL;
}

void KDCOnFlyPreviewJob::close()
{
	qDebug( "KDCOnFlyPreviewJob::closePreview()::begin" );
	setFlag( 3, FALSE );
	setFlag( 1, FALSE );
	setFlag( 2, FALSE );

	removeSubjob( p_mJob );

	if ( p_dlg )
	{
		p_dlg->disconnect();
		p_dlg->deleteLater();
	}
	emit done();
	qDebug( "KDCOnFlyPreviewJob::closePreview()::end" );
}

void KDCOnFlyPreviewJob::on_mplayerJobFinished( KDCJob *job )
{
	if ( job->hasErrors() )
		KMessageBox::error( 0, job->error() );
}
