/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcreadfileinfojob.h"

#include <QStringList>
#include <QDebug>

#include <KLocale>

KDCReadFileInfoJob::KDCReadFileInfoJob( const QString& file, QObject *parent )
	: KDCJob( parent )
{
	p_fileName = file;
	p_video = NULL;

	p_process = new KProcess( this );
	p_process->setOutputChannelMode( KProcess::MergedChannels );
	connect( p_process, SIGNAL( finished( int, QProcess::ExitStatus ) ), this, SLOT( on_finished( int, QProcess::ExitStatus ) ) );

	QStringList arg;
	arg << "-noautosub" << "-identify" << "-ao" << "null" << "-vo" << "null" << "-frames";
	arg << "1" << "-channels" << "6" << file;
	p_process->setProgram( "mplayer", arg );

	p_job = new KDCReadFileInfoSubjob( this );
	connect( p_job, SIGNAL( foundAudioStream( KDCAudioStreamMetaData * ) ), this, SLOT( on_foundAudioStream( KDCAudioStreamMetaData * ) ) );
	connect( p_job, SIGNAL( foundVideoStream( KDCVideoStreamMetaData * ) ), this, SLOT( on_foundVideoStream( KDCVideoStreamMetaData * ) ) );
	connect( p_job, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_subjobFinished( KDCJob * ) ) );
	connect( p_job, SIGNAL( foundAudioStreamID( const QList<int>& ) ), this, SLOT( on_foundAudioStreamID( const QList<int>& ) ) );

	p_process->setStandardOutputProcess( p_job->jobProcess() );
	setTotalSize( 100 );
}

KDCReadFileInfoJob::~KDCReadFileInfoJob()
{
	if ( p_job )
	{
		p_job->disconnect();
		delete p_job;
	}
	if ( p_process )
	{
		p_process->disconnect();
		if ( p_process->state() == QProcess::Running )
			p_process->close();
		delete p_process;
	}
}

void KDCReadFileInfoJob::start()
{
	qDebug("1");
	emit description( p_fileName );
	qDebug("2");
	emit infoMessage( i18n("Reading file info...") );
	qDebug("3");
	emit percent( 0 );
	qDebug("4");
	p_process->start();
	qDebug("5");
	p_job->start( KDCReadFileInfoSubjob::AudioVideo );
	qDebug("6");
}

void KDCReadFileInfoJob::on_foundAudioStreamID( const QList<int>& id )
{
	p_audioID = id;
}

void KDCReadFileInfoJob::on_foundAudioStream( KDCAudioStreamMetaData *audio )
{
	p_audio.append( audio );
}

void KDCReadFileInfoJob::on_foundVideoStream( KDCVideoStreamMetaData *video )
{
	p_video = video;
}

void KDCReadFileInfoJob::on_subjobFinished( KDCJob *job )
{
	qDebug() << "on_subjobFinished";
	if ( job->hasErrors() )
	{
		setError( job->error() );
		delete p_job;
		if ( p_video )
			delete p_video;
		while ( p_audio.isEmpty() )
			delete p_audio.takeFirst();
		emit finished( this );
	}
	else
	{
		if ( p_audioID.count() != p_audio.count() )
		{
			QStringList arg;
			arg = p_process->program();
			arg.removeFirst();
			if ( arg.count() == 11 )
			{
				arg.insert( 10, "-aid" );
				arg.insert( 11, "" );
			}
			int id = p_audioID.at( p_audio.count() );
			arg[ 11 ] = QString::number( id );
			p_process->setProgram( "mplayer", arg );
			p_process->start();
			p_job->start( KDCReadFileInfoSubjob::Audio, id );
		}
		else
		{
			if ( p_video )
				emit movieInfo( p_fileName, p_video, p_audio );
			else
				emit noVideoStreamFound();
			delete p_job; // p_job = job
			emit percent( 100 );
			emit finished( this );
		}
	}
}

void KDCReadFileInfoJob::on_finished( int code, QProcess::ExitStatus status )
{
	qDebug( "KDCReadFileInfoJob::finished()::codec = %d, status = %d", code, status );
	if ( status == QProcess::CrashExit && !flag( 1 ) )
	{
		setError( i18n("There was an error during process. Error codec is %1").arg( code ) );
		emit finished( this );
	}
}
