/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcsubtitleconvertjob.h"

#include "../kdcfilefactory.h"
#include "../kdcsubtitlestream.h"

#include <QByteArray>

#include <KLocale>

KDCSubtitleConvertJob::KDCSubtitleConvertJob( int id, const KDCMovie *movie, int delay, QObject *parent )
	: KDCJob( parent )
{
	p_id = id + 1;
	p_movie = movie;

	p_process = new KProcess( this );
	p_process->setOutputChannelMode( KProcess::MergedChannels );
	connect( p_process, SIGNAL( readyReadStandardOutput() ), this, SLOT( on_readyReadStandardOutput() ) );
	connect( p_process, SIGNAL( finished( int , QProcess::ExitStatus ) ), this, SLOT( on_finished( int , QProcess::ExitStatus ) ) );

	KDCSubtitleStream *sub = movie->subtitleStreamGroup()->stream( id );
	QStringList arg;
	arg << "-slave" << "-noautosub" << "-fontconfig" << "-subcp";
	arg << sub->textCodec() << "-subfont-encoding" << "unicode";
	arg << "-dumpmicrodvdsub" << "-nosound" << "-vo" << "null" << "-subdelay" << QString::number( -delay );
	arg << "-fps" << QString::number( sub->fps() ) << "-subfps" << QString::number( movie->fps() );
	arg << "-sub" << sub->fileName() << movie->fileName();

	p_process->setWorkingDirectory( KDCFileFactory::discDirectory( movie->disc() ) );
	p_process->setProgram( "mplayer", arg );

	setTotalSize( 0 );
	qDebug( QString( "KDCSubtitleConvertJob:: " + arg.join(" ") ).toLatin1() );
}

KDCSubtitleConvertJob::~KDCSubtitleConvertJob()
{
	if ( p_process )
	{
		disconnect( p_process, 0, 0, 0 );
		if ( p_process->state() == QProcess::Running )
			p_process->close();
		delete p_process;
	}
	qDebug( "~KDCSubtitleConvertJob::id = %d", p_id );
}

void KDCSubtitleConvertJob::start()
{
	emit jobTotalSize( 0 );
	emit percent( 0 );
	emit infoMessage( i18n("Start converting subtitle file ( %1 )").arg( p_id ) );
	qDebug( "KDCSubtitleConvertJob::start()::id = %d", p_id );
	p_process->start();

}

void KDCSubtitleConvertJob::on_readyReadStandardOutput()
{
	QByteArray tmp = p_process->readAllStandardOutput();
	p_data.append( tmp );
	if ( p_data.indexOf( "SUB: Subtitles dumped in" ) != -1 )
		p_process->write( QString("quit\n").toUtf8() );
	else
		if ( p_data.indexOf( "SUB: Could not determine file format" ) != -1 )
		{
			setError( i18n("Could not determine subtitles file format") );
			setFlag( 1, TRUE );
			p_process->close();
		}
	else
		if ( p_data.indexOf( "Cannot load subtitles:" ) != -1 )
		{
			setError( i18n("Cannot load subtitles file. Perhaps it's not subtitle file.") );
			setFlag( 1, TRUE );
			p_process->close();
		}
}

void KDCSubtitleConvertJob::on_finished( int code, QProcess::ExitStatus status )
{
	qDebug( "Codec = %d, status = %d", code, status );
	if ( status == QProcess::CrashExit && !flag( 1 ) )
	{
		setError( i18n("There was an error during process. Error code is %1").arg( code ) );
		KDCFileFactory::removeFile( KDCFileFactory::discDirectory( p_movie->disc() ) + "/dumpsub.sub" );
	}
	else
		if ( status == QProcess::CrashExit && flag( 1 ) )
			KDCFileFactory::removeFile( KDCFileFactory::discDirectory( p_movie->disc() ) + "/dumpsub.sub" );
	else
	{
		qDebug( QString( "KDCSubtitleConvertJob::rename(" + KDCFileFactory::discDirectory( p_movie->disc() ) + "/dumpsub.sub " + KDCFileFactory::movieSubtitleFile( p_movie, p_id ) ).toUtf8() );
		if ( !KDCFileFactory::renameFile(	KDCFileFactory::discDirectory( p_movie->disc() ) + "/dumpsub.sub",
																			KDCFileFactory::movieSubtitleFile( p_movie, p_id ) ) )
			setError( i18n("There was an error during process.") );
		else
			emit infoMessage( i18n("Converting subtitle file ( %1 ) sucesfully finished").arg( p_id ) );
	}
	qDebug( "KDCSubtitleConvertJob::end:: emit finished" );
	emit finished( this );
}
