/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcsubtitlecreatexmlfilejob.h"

#include "../kdcfilefactory.h"

#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QFile>
#include <QTextStream>

#include <KLocale>

KDCSubtitleCreateXMLFileJob::KDCSubtitleCreateXMLFileJob( int id, const KDCMovie *movie, QObject *parent )
	: KDCJob( parent )
{
	p_file = KDCFileFactory::movieSubtitleConfigFile( movie, id + 1 );
	KDCSubtitleCreateXMLFileJobPrivate( id, movie );
}

KDCSubtitleCreateXMLFileJob::KDCSubtitleCreateXMLFileJob( int id, const KDCMovieGroup *movie, QObject *parent )
	: KDCJob( parent )
{
	p_file = KDCFileFactory::movieSubtitleConfigFile( movie, id + 1 );
	KDCSubtitleCreateXMLFileJobPrivate( id, movie->KDCMovieGroup::movie( 0 ) );
}

void KDCSubtitleCreateXMLFileJob::KDCSubtitleCreateXMLFileJobPrivate( int id, const KDCMovie *movie )
{
	p_id = id + 1;
	KDCSubtitleStream *sub = movie->subtitleStreamGroup()->stream( id );

	p_doc = new QDomDocument;
	QDomElement subpictures = p_doc->createElement( "subpictures" );
	QDomElement stream = p_doc->createElement( "stream" );
	QDomElement textsub = p_doc->createElement( "textsub" );
	textsub.setAttribute( "characterset", "UTF-8" );
	textsub.setAttribute( "fontsize", sub->textSize() );
	textsub.setAttribute( "font", "kdcfont.ttf" );
	if ( movie->movieGroup() )
		textsub.setAttribute( "filename", KDCFileFactory::movieSubtitleFile( movie->movieGroup(), p_id ) );
	else
		textsub.setAttribute( "filename", KDCFileFactory::movieSubtitleFile( movie, p_id ) );
	textsub.setAttribute( "horizontal-alignment", "center" );
	textsub.setAttribute( "vertical-alignment", "bottom" );
	textsub.setAttribute( "left-margin", "60" );
	textsub.setAttribute( "right-margin", "60" );
	textsub.setAttribute( "top-margin", "20" );
	textsub.setAttribute( "bottom-margin", "40" );
	textsub.setAttribute( "movie-width", "720" );
	textsub.setAttribute( "subtitle-fps", movie->fps() );
	textsub.setAttribute( "movie-fps", movie->fps() );
	if ( movie->videoFormat() == KDC::PAL )
		textsub.setAttribute( "movie-height", "576" );
	else
		textsub.setAttribute( "movie-height", "480" );

	p_doc->appendChild( subpictures );
	subpictures.appendChild( stream );
	stream.appendChild( textsub );

	setTotalSize( 0 );
}

KDCSubtitleCreateXMLFileJob::~KDCSubtitleCreateXMLFileJob()
{
	delete p_doc;
}

void KDCSubtitleCreateXMLFileJob::start()
{
	emit jobTotalSize( 0 );
	emit infoMessage( i18n("Start creating xml file for subtitle stream ( %1 )").arg( p_id ) );
	emit percent( 0 );
	QFile out( p_file );
	if ( !out.open( QIODevice::WriteOnly | QIODevice::Truncate ) )
	{
		setError( i18n("There was an error during process.") );
		emit finished( this );
	}
	else
	{
		QTextStream st( &out );
		st << p_doc->toString();
		out.close();
		emit infoMessage( i18n("Creating xml file for subtitle stream ( %1 ) sucesfully finished").arg( p_id ) );
		emit finished( this );
	}
}
