/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcvcdxgencreatexmlfilejob.h"

#include "../kdcfilefactory.h"
#include "../kdcnamespace.h"
#include "../kdcmovie.h"
#include "../kdcmoviegroup.h"

#include <QList>
#include <QTreeWidgetItem>

#include <KLocale>

KDCVCDXGenCreateXMLFileJob::KDCVCDXGenCreateXMLFileJob( KDCXVCDDisc *disc, QObject *parent )
	: KDCJob( parent )
{
	p_process = new KProcess( this );
	p_process->setOutputChannelMode( KProcess::MergedChannels );
	connect( p_process, SIGNAL( readyReadStandardOutput() ), this, SLOT( on_readyReadStandardOutput() ) );
	connect( p_process, SIGNAL( finished( int , QProcess::ExitStatus ) ), this, SLOT( on_finished( int , QProcess::ExitStatus ) ) );

	QStringList arg;
	arg << "-t";
	if ( disc->discType() == KDC::VCD )
		arg << "vcd2";
	else
		arg << "svcd";
	arg << "-o" << KDCFileFactory::discConfigFile( disc );

	KDCMovie *movie;
	KDCMovieGroup *group;
	QList<QTreeWidgetItem*> list = disc->takeMovies();
	QList<QTreeWidgetItem*>::const_iterator it;
	for ( it = list.constBegin(); it != list.constEnd(); ++it )
	{
		if ( ( *it )->type() == KDC::Movie )
		{
			movie = static_cast<KDCMovie*>( *it );
			arg << KDCFileFactory::movieName( movie );
		}
		else
		if ( ( *it )->type() == KDC::MovieGroup )
		{
			group = static_cast<KDCMovieGroup*>( *it );
			arg << KDCFileFactory::movieName( group );
		}
	}

	p_process->setWorkingDirectory( KDCFileFactory::discDirectory( disc ) );
	p_process->setProgram( "vcdxgen", arg );

	setTotalSize( 0 );
	qDebug( QString( "KDCVCDXGenCreateXMLFileJob:: " + arg.join(" ") ).toUtf8() );
}

KDCVCDXGenCreateXMLFileJob::~KDCVCDXGenCreateXMLFileJob()
{
	if ( p_process )
	{
		disconnect( p_process, 0, 0, 0 );
		if ( p_process->state() == QProcess::Running )
			p_process->close();
		delete p_process;
	}
}

void KDCVCDXGenCreateXMLFileJob::start()
{
	emit jobTotalSize( 0 );
	emit percent( 0 );
	emit infoMessage( i18n("Start creating config file for disc.") );
	p_process->start();
}

void KDCVCDXGenCreateXMLFileJob::on_readyReadStandardOutput()
{
	qDebug( p_process->readAllStandardOutput() );
}

void KDCVCDXGenCreateXMLFileJob::on_finished( int code, QProcess::ExitStatus status )
{
	if ( status == QProcess::CrashExit )
		setError( i18n("There was an error during process. Error code is %1" ).arg( code ) );
	else
		emit infoMessage( i18n("Creating config file for disc sucesfully finished") );
	emit finished( this );
}
