/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcchaptergroup.h"

#include "kdcmovie.h"
#include "kdcmoviegroup.h"
#include "lib/kdclib.h"
#include "kdcfilefactory.h"

#include <QTime>

KDCChapterGroup::KDCChapterGroup( KDCMovie *movie )
{
	p_movie = movie;
	p_group = NULL;
	p_mode = KDC::List;
}

KDCChapterGroup::KDCChapterGroup( KDCMovieGroup *movie )
{
	p_group = movie;
	p_movie = NULL;
	p_mode = KDC::List;
}

KDCChapterGroup::KDCChapterGroup( const KDCChapterGroup* group )
{
	//todo
}

KDCChapterGroup::~KDCChapterGroup()
{
	if ( p_movie )
		KDCFileFactory::removeDirectory( KDCFileFactory::movieDir( p_movie ) );
	else
		KDCFileFactory::removeDirectory( KDCFileFactory::movieDir( p_group ) );
}

int KDCChapterGroup::count() const
{
	return p_data.count();
}

void KDCChapterGroup::clear()
{
	p_data.clear();
}

void KDCChapterGroup::createFromLength( int delta )
{
	int i;
	double length;
	if ( p_movie )
		length = p_movie->videoStream()->metaData()->length()/1000.0;
	else
	{
		for ( i=0; i<p_group->count(); ++i )
			length += p_group->movie( i )->videoStream()->metaData()->length();
		length /= 1000.0;
	}
	int size = qRound( length / delta );
	p_data.clear();
	for ( i=0; i<size; ++i )
		p_data.append( delta*( i+1 ) );
}

void KDCChapterGroup::createFromSize( int size )
{
	int i;
	double length;
	if ( p_movie )
		length = p_movie->videoStream()->metaData()->length()/1000.0;
	else
	{
		for ( i=0; i<p_group->count(); ++i )
			length += p_group->movie( i )->videoStream()->metaData()->length();
		length /= 1000.0;
	}
	int delta = qRound( length/( size +1 ) );
	p_data.clear();
	for ( i=0; i<size; ++i )
		p_data.append( delta*( i+1 ) );
}

int KDCChapterGroup::chapter( int index )
{
	return p_data.at( index );
}

void KDCChapterGroup::sort()
{
	qSort( p_data.begin(), p_data.end() );
}

void KDCChapterGroup::add( int chapter )
{
	if ( p_data.indexOf( chapter ) == -1 )
	{
		p_data.append( chapter );
		sort();
	}
}

void KDCChapterGroup::remove( int index )
{
	int max = p_data.size() - 1;
	if ( max < 0 )
		max = 0;
	if ( KDCLib::isInRange( index, 0, max ) ) 
		p_data.removeAt( index );
}

KDC::ViewMode KDCChapterGroup::viewMode() const
{
	return p_mode;
}

void KDCChapterGroup::setViewMode( KDC::ViewMode mode )
{
	p_mode = mode;
}

QString KDCChapterGroup::chapterAsString( int index, const QString& format ) const
{
	QTime time;
	return time.addSecs( p_data.at( index ) ).toString( format );
}
