/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcdiscfactory.h"

#include "kdcdvddisc.h"
#include "kdcsvcddisc.h"
#include "kdcvcddisc.h"
#include "kdccvddisc.h"
#include "kdcconfig.h"
#include "lib/kdclib.h"

#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QDir>
#include <QVariant>

#include <KLocale>
#include <KIcon>
#include <KMessageBox>

KDCDiscFactory::KDCDiscFactory( QWidget *parent )
	: KTabWidget( parent )
{
	setHoverCloseButton( TRUE );

	p_currentDisc = NULL;
	p_discID = 0;

	connect( this, SIGNAL( closeRequest( QWidget * ) ), this, SLOT( on_closeRequest( QWidget * ) ) );
	connect( this, SIGNAL( currentChanged( int ) ), this, SLOT( on_currentChanged( int ) ) );
	setVisible( FALSE );
}

KDCDiscFactory::~KDCDiscFactory()
{

}

KDCDisc* KDCDiscFactory::currentDisc() const
{
	return p_currentDisc;
}

void KDCDiscFactory::on_closeRequest( QWidget *widget )
{
	qDebug( "KDCDiscFactory::closeRequest():: emit needToCloseDisc()" );
	emit needToCloseDisc( static_cast<KDCDisc*>( widget ) );
}

void KDCDiscFactory::closeDisc( KDCDisc *disc )
{
	qDebug( "KDCDiscFactory::closeDisc()::begin" );
	removeTab( indexOf( disc ) );
	if ( count() == 0 )
	{
		p_currentDisc = NULL;
		setVisible( FALSE );
		qDebug( "KDCDiscFactory::closeDisc()::emit currentDiscChanged( NULL )" );
		emit currentDiscChanged( NULL );
	}
	if ( disc->discType() == KDC::DVD )
		delete static_cast<KDCDVDDisc*>( disc );
	else
		if ( disc->discType() == KDC::CVD )
			delete static_cast<KDCCVDDisc*>( disc );
	else
		if ( disc->discType() == KDC::SVCD )
			delete static_cast<KDCSVCDDisc*>( disc );
	else
		delete static_cast<KDCVCDDisc*>( disc );
	qDebug( "KDCDiscFactory::closeDisc()::end" );
}

KDCDisc* KDCDiscFactory::getDisc( int index ) const
{
	if ( index == -1 )
		return NULL;
	if ( index < count() )
		return static_cast<KDCDisc*>( widget( index ) );

	return NULL;
}

void KDCDiscFactory::addNewDisc( KDC::DiscType type )
{
	int index;
	p_discID++;

	if ( type == KDC::DVD )
		index = addTab( new KDCDVDDisc( p_discID, this ), KIcon("media-optical"), i18n("DVD Video Disc") );
	else
		if ( type == KDC::CVD )
			index = addTab( new KDCCVDDisc( p_discID, this ), KIcon("media-optical"), i18n("CVD Video Disc") );
	else
		if ( type == KDC::SVCD )
			index = addTab( new KDCSVCDDisc( p_discID, this ), KIcon("media-optical"), i18n("SVCD Video Disc") );
	else
		index = addTab( new KDCVCDDisc( p_discID, this ), KIcon("media-optical"), i18n("VCD Video Disc") );

	KDCDisc *disc = getDisc( index );
	connect( disc, SIGNAL( objectChanged( QTreeWidgetItem * ) ), this, SLOT( on_objectChanged( QTreeWidgetItem * ) ) );
	connect( disc, SIGNAL( customContextMenuRequested( const QPoint & ) ), this, SLOT( on_customContextMenuRequested( const QPoint & ) ) );
	connect( disc, SIGNAL( settingsChanged() ), this, SLOT( on_discSettingsChanged() ) );
	connect( disc, SIGNAL( dropedSomeFiles( const QStringList& ) ), this, SLOT( on_dropedSomeFiles( const QStringList& ) ) );

	setCurrentIndex( index );
	if ( !isVisible() )
		setVisible( TRUE );
	on_currentChanged( index );
}

void KDCDiscFactory::on_objectChanged( QTreeWidgetItem *item )
{
	qDebug( "KDCDiscFactory::currentItemChanged()::begin::item = %d", KDCLib::isNull( item ) );
	emit currentObjectChanged( item );
	if ( item )
	{
		bool canUp = FALSE, canDown = FALSE;
		QTreeWidgetItem *parent;

		if ( p_currentDisc->discType() == KDC::DVD && item->type() != KDC::VTS )
		{
			parent = item->parent();
			int index = parent->indexOfChild( item );
			if ( index > 0 )
			{
				if ( parent->child( 0 )->type() == KDC::VMGMenu )
					canUp = FALSE;
				else
					canUp = TRUE;
			}
			if ( index + 1 < parent->childCount() )
				canDown = TRUE;
		}
		else
		{
			int index = p_currentDisc->indexOfTopLevelItem( item );
			if ( index > 0 )
			{
				parent = p_currentDisc->itemAbove( item );
				if ( parent )
				{
					if ( parent->type() == KDC::VMGMenu )
						canUp = FALSE;
					else
						canUp = TRUE;
			}
			else
				canUp = FALSE;
			}
			if( index + 1 < p_currentDisc->topLevelItemCount() )
				canDown = TRUE;
		}
		emit canMoveUpDown( canUp, canDown );
	}
	qDebug( "KDCDiscFactory::currentItemChanged()::end" );
}

void KDCDiscFactory::on_customContextMenuRequested( const QPoint & pos )
{
	qDebug( "KDCDiscFactory::currentItemChanged():: emit currentDiscContextMenu()" );
	emit currentDiscContextMenu( currentDisc()->mapToGlobal( pos ) );
}

void KDCDiscFactory::on_discSettingsChanged()
{
	qDebug( "KDCDiscFactory::discSizeChanged()" );
	emit currentDiscSettingsChanged( p_currentDisc->settings()->mediaSize(), p_currentDisc->size(), p_currentDisc->settings()->outputPath() );
}

void KDCDiscFactory::on_currentChanged( int index )
{
	qDebug( "KDCDiscFactory::currentChanged()::begin, index = %d", index );
	p_currentDisc = getDisc( index );
	qDebug( "KDCDiscFactory::currentItemChanged():: emit currentDiscChanged( %d )", KDCLib::isNull( p_currentDisc ) );
	emit currentDiscChanged( p_currentDisc );
	on_objectChanged( p_currentDisc->currentItem() );
	qDebug( "KDCDiscFactory::currentChanged()::end" );
}

void KDCDiscFactory::on_dropedSomeFiles( const QStringList& files )
{
	qDebug( "KDCDiscFactory::currentItemChanged():: emit needToOpenSomeFiles( files, %d )", KDCLib::isNull( p_currentDisc ) );
	emit needToOpenSomeFiles( files );
}

KMenu* KDCDiscFactory::currentDiscMenu() const
{
	if ( p_currentDisc )
		return p_currentDisc->settings()->menu();
	return NULL;
}

