/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef KDCMOVIE_H
#define KDCMOVIE_H

#include "kdcnamespace.h"
#include "kdcaudiostreamgroup.h"
#include "kdcsubtitlestreamgroup.h"
#include "kdcmoviegroup.h"
#include "kdcvideostreammetadata.h"
#include "kdcaudiostreammetadata.h"
#include "kdcvideostream.h"
#include "kdcchaptergroup.h"

#include <QTreeWidgetItem>
#include <QString>
#include <QObject>

class KDCDisc;
class KDCMovieGroup;

class KDCMovie : public QObject, public QTreeWidgetItem
{
Q_OBJECT
public:
	KDCMovie( const QString& fileName,
						KDCVideoStreamMetaData *videoStream,
						const QList<KDCAudioStreamMetaData*>& audioStreams );
	KDCMovie( KDCMovie *movie );
	void copySettings( const KDCMovie *movie );
	~KDCMovie();
	KDCDisc* disc() const;
	int id() const;
	KDCMovieGroup* movieGroup() const;

	void checkSubtitles();

	KDCVideoStream* videoStream() const;
	KDCAudioStreamGroup* audioStreamGroup() const;
	KDCSubtitleStreamGroup* subtitleStreamGroup() const;
	KDCChapterGroup* chapterGroup() const;

	KDC::VideoFormat videoFormat() const;
	void setVideoFormat( KDC::VideoFormat format );
	void setFps( double fps );
	double fps() const;

	QString fileName() const;
	QString outputName() const;
	void setOutputName( const QString& name );

	quint64 size() const;
	void setUpdatesMode( KDC::Mode mode );
	KDC::Mode updatesMode() const;

	bool operator!=( const KDCMovie& );
public slots:
	void refresh();
signals:
	void settingsChanged();
private:
	friend class KDCDisc;
	friend class KDCDVDDisc;
	friend class KDCVCDDisc;
	friend class KDCSVCDDisc;
	friend class KDCCVDDisc;
	friend class KDCXVCDDisc;
	friend class KDCMovieGroup;
	KDC::VideoFormat p_format;
	QString p_fileName;
	QString p_outputName;
	quint64 p_size;
	KDC::Mode p_updates;
	KDCAudioStreamGroup *p_audioGroup;
	KDCSubtitleStreamGroup *p_subGroup;
	KDCVideoStream *p_videoStream;
	KDCChapterGroup *p_chapterGroup;
	double p_fps;
	int p_id;
	KDCMovieGroup *p_group;
	KDCDisc *p_disc;
};
#endif
