/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcvideostream.h"

#include "kdcdisc.h"
#include "lib/kdclib.h"

#include <KLocale>

KDCVideoStream::KDCVideoStream( KDCMovie *movie, KDCVideoStreamMetaData *metaData )
	: QObject( 0 )
{
	p_movie = movie;
	p_metaData = metaData;
	p_mode = KDC::Resample;

	p_trellisMode = KDC::Disabled;
	p_deinterlaceMode = KDC::None;
	if ( p_metaData->demuxer() == "real" )
		p_macroBlockMode = KDC::MDB2;
	else
		p_macroBlockMode = KDC::MBCMP;
	p_hq = KDC::Disabled;
	p_gop = KDC::Disabled;
	p_updates = KDC::Disabled;
	p_ar = KDC::LetterBox;
	p_bitrate = 0;
	p_codec = KDC::mpeg2;
}

KDCVideoStream::~KDCVideoStream()
{
	if ( p_metaData )
		delete p_metaData;
}

void KDCVideoStream::copySettings( const KDCVideoStream *stream )
{
	p_mode = stream->mode();
	p_trellisMode = stream->trellisMode();
	p_deinterlaceMode = stream->deinterlaceMode();
	p_macroBlockMode = stream->macroBlockMode();
	p_hq = stream->hqDVDMode();
	p_gop = stream->closestGOPMode();
	p_bitrate = stream->bitrate();
	p_width = stream->width();
	p_height = stream->height();
	p_codec = stream->codec();
	p_ar = stream->aspectRatio();
	refresh();
}

KDCVideoStreamMetaData* KDCVideoStream::metaData() const
{
	return p_metaData;
}

KDCVideoStream::KDCVideoStream( KDCVideoStream *stream )
	: QObject( 0 )
{
	p_metaData = new KDCVideoStreamMetaData(	stream->metaData()->demuxer(), stream->metaData()->codec(),
																						stream->metaData()->width(), stream->metaData()->height(),
																						stream->metaData()->bitrate(), stream->metaData()->fps(),
																						stream->metaData()->length(), stream->metaData()->aspectRatio() );
	p_bitrate = stream->bitrate();
	p_ar = stream->aspectRatio();
	p_macroBlockMode = stream->macroBlockMode();
	p_deinterlaceMode = stream->deinterlaceMode();
	p_trellisMode = stream->trellisMode();
	p_width = stream->width();
	p_height = stream->height();
	p_mode = stream->mode();
	p_hq = stream->hqDVDMode();
	p_codec = stream->codec();
	p_updates = stream->updatesMode();
}

void KDCVideoStream::setBitrate( int bitrate )
{
	if ( p_bitrate != bitrate )
	{
		qDebug( "KDCVideoStream::setBitrate( %d )", bitrate );
		p_bitrate = bitrate;
		if ( p_movie->disc()->discType() == KDC::DVD && p_codec == KDC::mpeg2 && p_hq == KDC::Enabled && p_bitrate < 4500 )
			p_hq = KDC::Disabled;
		if ( p_mode == KDC::Copy && bitrate != p_metaData->bitrate() )
			setMode( KDC::Resample );
		else
			refresh();
	}
}

void KDCVideoStream::setAspectRatio( KDC::AspectRatio aspect )
{
	if ( p_ar != aspect )
	{
		qDebug( "KDCVideoStream::setAspectRatio( %d )", aspect );
		p_ar = aspect;
		if ( p_ar == KDC::WideScreen && p_movie->disc()->discType() == KDC::DVD && p_codec == KDC::mpeg2 )
		{
			p_width = 720;
			if ( p_movie->videoFormat() == KDC::PAL )
				p_height = 576;
			else
				p_height = 480;
		}
		if ( p_hq == KDC::Enabled && p_ar != KDC::WideScreen )
			p_hq = KDC::Disabled;
		if ( p_mode == KDC::Copy && (	( aspect == KDC::LetterBox && p_metaData->aspectRatio() != 1.333 ) ||
																	( aspect == KDC::WideScreen && p_metaData->aspectRatio() != 1.78 ) ) )
			setMode( KDC::Resample );
		else
			refresh();
	}
}

void KDCVideoStream::setMacroBlockMode( KDC::MacroBlockMode mode )
{
	p_macroBlockMode = mode;
	if ( p_movie->movieGroup() )
		emit settingsChanged();
}

void KDCVideoStream::setDeinterlaceMode( KDC::DeinterlaceMode mode )
{
	p_deinterlaceMode = mode;
	if ( p_movie->movieGroup() )
		emit settingsChanged();
}

void KDCVideoStream::setTrellisMode( KDC::Mode mode )
{
	p_trellisMode = mode;
	if ( p_movie->movieGroup() )
		emit settingsChanged();
}

void KDCVideoStream::setResolution( int width, int height )
{
	if ( p_width != width || p_height != height )
	{
		qDebug( "KDCVideoStream::setResolution( %d, %d )", width, height );
		p_width = width;
		p_height = height;
		if ( p_movie->disc()->discType() == KDC::DVD && p_codec == KDC::mpeg2 )
		{
			if ( p_width != 720 )
			{
				p_hq = KDC::Disabled;
				p_ar = KDC::LetterBox;
			}
		}
		if ( p_mode == KDC::Copy && ( width != p_metaData->width() || height != p_metaData->height() ) )
			setMode( KDC::Resample );
		else
			refresh();
	}
}

int KDCVideoStream::bitrate() const
{
	return p_bitrate;
}

int KDCVideoStream::width() const
{
	return p_width;
}

int KDCVideoStream::height() const
{
	return p_height;
}

KDC::AspectRatio KDCVideoStream::aspectRatio() const
{
	return p_ar;
}

KDC::MacroBlockMode KDCVideoStream::macroBlockMode() const
{
	return p_macroBlockMode;
}

KDC::DeinterlaceMode KDCVideoStream::deinterlaceMode() const
{
	return p_deinterlaceMode;
}

KDC::Mode KDCVideoStream::trellisMode() const
{
	return p_trellisMode;
}

KDC::Mode KDCVideoStream::closestGOPMode() const
{
	return p_gop;
}

void KDCVideoStream::setClosestGOPMode( KDC::Mode mode )
{
	p_gop = mode;
	if ( p_movie->movieGroup() )
		emit settingsChanged();
}

KDC::StreamMode KDCVideoStream::mode() const
{
	return p_mode;
}

void KDCVideoStream::setMode( KDC::StreamMode mode )
{
	if ( p_mode != mode )
	{
		qDebug( "KDCVideoStream::setMode( %d )", mode );
		p_mode = mode;
		if ( p_mode == KDC::Copy )
		{
			if ( p_metaData->fps() == 25 )
				p_movie->setVideoFormat( KDC::PAL );
			else
				p_movie->setVideoFormat( KDC::NTSC );
			p_width = metaData()->width();
			p_height = metaData()->height();
			p_bitrate = metaData()->bitrate();
			p_hq = KDC::Disabled;
			p_ar = KDCLib::aspectRatio( p_movie );
			if ( p_metaData->codec() == "0x10000002" )
				p_codec = KDC::mpeg2;
			else
				p_codec = KDC::mpeg1;
		}
		refresh();
	}
}

void KDCVideoStream::refresh()
{
	if ( p_updates == KDC::Enabled )
	{
		// display
		QStringList values;
		if ( p_movie->videoFormat() == KDC::PAL )
			values.append( "PAL" );
		else
			values.append( "NTSC" );
		values.append( QString::number( p_movie->fps(), 'f', 3 ) );
		values.append( QString::number( p_bitrate ) );
		values.append( QString::number( p_width ) + "x" +  QString::number( p_height ) );
		if ( p_ar == KDC::LetterBox )
			values.append( "4:3" );
		else
			values.append( "16:9" );

		p_size = qRound( 0.000125*p_metaData->length()*p_bitrate );
		p_info = QString( "\tv\t" + values.join("\t") );
		qDebug( "KDCVideoStream::refresh():: emit settingsChanged()" );
		emit settingsChanged();
	}
}

quint64 KDCVideoStream::size() const
{
	return p_size;
}

void KDCVideoStream::setHQDVDMode( KDC::Mode mode )
{
	if ( p_hq != mode )
	{
		qDebug( "KDCVideoStream::setHQDVDMode( %d )", mode );
		p_hq = mode;
		if ( p_movie->disc()->discType() == KDC::DVD && p_hq == KDC::Enabled )
		{
			p_codec = KDC::mpeg2;
			p_width = 720;
			if ( p_movie->videoFormat() == KDC::PAL )
				p_height = 576;
			else
				p_height = 480;
			if ( p_bitrate < 4500 )
				p_bitrate = 4500;
		}
		refresh();
	}
}

KDC::Mode KDCVideoStream::hqDVDMode() const
{
	return p_hq;
}

bool KDCVideoStream::operator!=( const KDCVideoStream&  stream )
{
	if ( this && &stream )
	{
		if (	p_bitrate != stream.bitrate() )
			return TRUE;
		if( p_ar != stream.aspectRatio() )
			return TRUE;
		if ( p_width != stream.width() )
			return TRUE;
		if ( p_height != stream.height() )
			return TRUE;
		if ( p_macroBlockMode != stream.macroBlockMode() )
			return TRUE;
		if ( p_deinterlaceMode != stream.deinterlaceMode() )
			return TRUE;
		if ( p_trellisMode != stream.trellisMode() )
			return TRUE;
		if ( p_hq != stream.hqDVDMode() )
			return TRUE;
		if ( p_codec != stream.codec() )
			return TRUE;
		return FALSE;
	}
	else
		if ( ( this != NULL && &stream == NULL ) || ( this == NULL && &stream != NULL ) )
			return TRUE;

	return FALSE;
}

void KDCVideoStream::setCodec( KDC::VideoCodec codec )
{
	if ( p_codec != codec )
	{
		qDebug( "KDCVideoStream::setCodec( %d )", codec );
		p_codec = codec;
		if ( p_movie->disc()->discType() == KDC::DVD && p_codec == KDC::mpeg1 )
		{
			if ( p_bitrate > 1856 )
				p_bitrate = 1856;
			p_hq = KDC::Disabled;
			p_width = 352;
			if ( p_movie->videoFormat() == KDC::PAL )
				p_height = 288;
			else
				p_height = 240;
		}
		else
			if ( p_movie->disc()->discType() == KDC::DVD && p_codec == KDC::mpeg2 )
				p_ar = KDC::LetterBox;
		if ( p_mode == KDC::Copy && ( ( codec == KDC::mpeg1 && p_metaData->codec() != "0x10000001" ) ||
																	( codec == KDC::mpeg2 && p_metaData->codec() != "0x10000002" ) ) )
			setMode( KDC::Resample );
		else
			refresh();
	}
}

KDC::VideoCodec KDCVideoStream::codec() const
{
	return p_codec;
}

void KDCVideoStream::setUpdatesMode( KDC::Mode mode )
{
	if ( mode != p_updates )
		p_updates = mode;
}

KDC::Mode KDCVideoStream::updatesMode() const
{
	return p_updates;
}

QString KDCVideoStream::info() const
{
	return p_info;
}

bool KDCVideoStream::isSVCDPALVideoStream( const KDCVideoStreamMetaData *stream )
{
	if (	stream->demuxer() == "mpegps" && stream->fps() == 25.0 && stream->width() == 480 &&
				stream->height() == 576 && stream->codec() == "0x10000002"  && stream->bitrate() <= 2600 )
		return TRUE;
	else
		return FALSE;
}

bool KDCVideoStream::isSVCDNTSCVideoStream( const KDCVideoStreamMetaData *stream )
{
	if	(	stream->demuxer() == "mpegps" && stream->fps() == 29.97 && stream->width() == 480 &&
				stream->height() == 480 && stream->codec() == "0x10000002"  && stream->bitrate() <= 2600 )
		return TRUE;
	else
		return FALSE;
}

bool KDCVideoStream::isVCDPALVideoStream( const KDCVideoStreamMetaData *stream )
{
	if (	stream->demuxer() == "mpegps" && stream->fps() == 25.0 && stream->width() == 352 &&
				stream->height() == 288 && stream->codec() == "0x10000001"  && stream->bitrate() == 1152 )
		return TRUE;
	else
		return FALSE;
}

bool KDCVideoStream::isVCDNTSCVideoStream( const KDCVideoStreamMetaData *stream )
{
	if	(	stream->demuxer() == "mpegps" && ( stream->fps() == 29.97 || stream->fps() == 23.976 ) &&
				stream->width() == 352 && stream->height() == 240 && stream->codec() == "0x10000001" &&
				stream->bitrate() == 1152 )
		return TRUE;
	else
		return FALSE;
}

bool KDCVideoStream::isDVDPALVideoStream( const KDCVideoStreamMetaData *stream )
{
	if (	stream->demuxer() == "mpegps" && stream->fps() == 25.0 &&
				KDCVideoStream::isPALDVDResolution( stream->width(), stream->height() ) &&
				( ( stream->codec() == "0x10000002" && stream->bitrate() <= 9800 ) ||
					( stream->codec() == "0x10000001" && stream->bitrate() <= 1856 ) ) )
		return TRUE;
	else
		return FALSE;
}

bool KDCVideoStream::isDVDNTSCVideoStream( const KDCVideoStreamMetaData *stream )
{
	if (	stream->demuxer() == "mpegps" && ( stream->fps() == 29.97 || stream->fps() == 23.976 ) &&
				KDCVideoStream::isNTSCDVDResolution( stream->width(), stream->height() ) &&
				( ( stream->codec() == "0x10000002" && stream->bitrate() <= 9800 ) ||
					( stream->codec() == "0x10000001" && stream->bitrate() <= 1856 ) ) )
		return TRUE;
	else
		return FALSE;
}

bool KDCVideoStream::isCVDPALVideoStream( const KDCVideoStreamMetaData *stream )
{
	if (	stream->demuxer() == "mpegps" && stream->fps() == 25.0 && stream->width() == 352 &&
				stream->height() == 576 && stream->codec() == "0x10000002" && stream->bitrate() <= 2600 )
		return TRUE;
	else
		return FALSE;
}

bool KDCVideoStream::isCVDNTSCVideoStream( const KDCVideoStreamMetaData *stream )
{
	if	(	stream->demuxer() == "mpegps" && stream->fps() == 29.97 && stream->width() == 352 &&
				stream->height() == 480 && stream->codec() == "0x10000002" && stream->bitrate() <= 2600 )
		return TRUE;
	else
		return FALSE;
}

bool KDCVideoStream::isPALDVDResolution( int width, int height )
{
	if ( height == 576 )
	{
		if ( width == 720 || width == 704 || width == 352 )
			return TRUE;
		else
			return FALSE;
	}	
	else
		if ( height == 288 && width == 352 )
			return TRUE;
	else
		return FALSE;
}

bool KDCVideoStream::isNTSCDVDResolution( int width, int height )
{
	if ( height == 480 )
	{
		if ( width == 720 || width == 704 || width == 352 )
			return TRUE;
		else
			return FALSE;
	}	
	else
		if ( height == 240 && width == 352 )
			return TRUE;
	else
		return FALSE;
}
