/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "widgets/kdcdiscsizestatusbar.h"
#include "widgets/kdcoutputdirstatusbar.h"
#include "widgets/kdcinfostatusbar.h"
#include "kdcdiscfactory.h"
#include "kdcdvddisc.h"
#include "widgets/kdcdiscoptionsstatusbar.h"

#include <QVector>
#include <QTreeWidgetItem>
#include <QStringList>
#include <QMap>

#include <KXmlGuiWindow>
#include <KAction>
#include <KActionMenu>
#include <KSelectAction>

/**
	@author Marcin Swiechowicz <marcin.swiechowicz@gmail.com>
*/

class MainWindow : public KXmlGuiWindow
{
Q_OBJECT
public:
	MainWindow( QWidget *parent = 0 );
	~MainWindow();
private slots:
	void on_addMovieAction_triggered();
	void on_addVMGMenuAction_triggered();
	void on_addDVDDiscAction_triggered();
	void on_addCVDDiscAction_triggered();
	void on_addSVCDDiscAction_triggered();
	void on_addVCDDiscAction_triggered();
	void on_upAction_triggered();
	void on_downAction_triggered();
	void on_deleteAction_triggered();
	void preferences();
	void closePreview();
	void on_quit();
	void on_addVTSAction_triggered();
	void on_previewAction_triggered();
	void on_destroyGroupAction_triggered();
	void on_createGroupAction_triggered();
	void on_addToGroupAction_triggered();
	void on_removeFromGroupAction_triggered();
	void on_currentDiscContextMenu( const QPoint& );
	void on_needToCloseDisc( KDCDisc * );
	void on_currentObjectChanged( QTreeWidgetItem *item );
	void on_canMoveUpDown( bool canUp, bool canDown );
	void on_currentDiscChanged( KDCDisc * );
	void on_buildProjectAction_triggered();
	void on_buildAllProjectsAction_triggered();
	void on_currentDiscSettingsChanged( KDC::MediaSize , quint64 , const QString& );
	void on_readingFilesFinished( bool );
	void on_processingDiscsFinished();
	void on_separateVTSAction_triggered();
	void openFiles( const QStringList& );
	void on_deleteFilesStatusChanged( bool );
	void on_deleteTempFilesStatusChanged( bool );
	void on_structureStatusChanged( bool );
private:
	KDCDiscSizeStatusBar *discSizeStatus;
	KDCOutputDirStatusBar *outputDirStatus;
	KDCInfoStatusBar *infoStatus;
	KDCDiscFactory *discFactory;
	KDCDiscOptionsStatusBar *discOptionsStatus;

	KAction *quitAction;
	KActionMenu *addDiscAction;
	KAction *addDVDDiscAction;
	KAction *addSVCDDiscAction;
	KAction *addCVDDiscAction;
	KAction *addVCDDiscAction;
	KAction *addHDDVDDiscAction;
	KAction *addBDDiscAction;
	KAction *addVMGMenuAction;
	KAction *addMovieAction;
	KAction *addVTSAction;
	KActionMenu *addObjectAction;
	KAction *buildProjectAction;
	KAction *buildAllProjectsAction;
	KActionMenu *dvdSchemeAction;
	KAction *separateVTSAction;

	KAction *upAction;
	KAction *downAction;
	KAction *deleteAction;
	KActionMenu *createGroupAction;
	KActionMenu *addToGroupAction;
	KActionMenu *removeFromGroupAction;
	KAction *previewAction;
	KAction *destroyGroupAction;

	QVector<KAction*> movieActions;
	QVector<KAction*> groupActions;

	void createActions();
	void addActionsToCollection();
	void preparePopupMenu();
	void createStatusBar();
	void prepareVTSActions( KDCDVDDisc * disc );
};
#endif
