/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcencodingcombobox.h"

#include <QString>
#include <QStringList>
#include <QTextCodec>

#include <KGlobal>
#include <KCharsets>
#include <KLocale>

KDCEncodingComboBox::KDCEncodingComboBox( const QString& encoding, QWidget *parent )
	: KComboBox( parent )
{
	QString sEncoding = encoding;
	QString systemEncoding = QLatin1String( KGlobal::locale()->encoding() );
	if ( sEncoding.isEmpty() )
		sEncoding = systemEncoding;

	QStringList encodings( KGlobal::charsets()->availableEncodingNames() );
	int insert = 0;
	int system = 0;
	bool foundRequested = false;
	for ( int i = 0; i<encodings.count(); i++ )
	{
		bool found = false;
		QTextCodec *codecForEnc = KGlobal::charsets()->codecForName( encodings[i], found );
		if ( found )
		{
			addItem( encodings[i] );
			if ( ( codecForEnc->name() == sEncoding ) || ( encodings[i] == sEncoding ) )
			{
				setCurrentIndex( insert );
				foundRequested = true;
			}
			if ( ( codecForEnc->name() == systemEncoding ) || ( encodings[i] == systemEncoding ) )
				system = insert;
			insert++;
		}
	}

	if ( !foundRequested )
		setCurrentIndex( system );

	setCurrentItem( encoding );
	connect( this, SIGNAL( currentIndexChanged( int ) ), this, SLOT( on_currentIndexChanged( int ) ) );
}

KDCEncodingComboBox::~KDCEncodingComboBox()
{

}

QString KDCEncodingComboBox::currentCodec() const
{
	return currentText();
}

void KDCEncodingComboBox::setCurrentCodec( const QString& codec )
{
	setCurrentItem( codec );
}

void KDCEncodingComboBox::on_currentIndexChanged( int )
{
	emit currentCodecChanged( currentCodec() );
}
