/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcmovielist.h"

#include "../../lib/kdclib.h"

#include <QUrl>
#include <QDrag>
#include <QMimeData>
#include <QHeaderView>

#include <KIcon>
#include <KLocale>

KDCMovieList::KDCMovieList( QWidget *parent )
	: QTreeWidget( parent )
{
	setAlternatingRowColors( FALSE );
	setColumnCount( 1 );
	setSelectionBehavior( QAbstractItemView::SelectRows );
	setAcceptDrops( TRUE );
	setDropIndicatorShown( TRUE );
	setRootIsDecorated( FALSE );

	QStringList headers;
	headers << i18n("Movie Name");
	setHeaderLabels( headers );
	header()->setDefaultAlignment( Qt::AlignHCenter | Qt::AlignVCenter );

	connect( this, SIGNAL( currentItemChanged( QTreeWidgetItem *, QTreeWidgetItem * ) ), this, SLOT( on_currentItemChanged( QTreeWidgetItem *, QTreeWidgetItem * ) ) );

	clean();
}

KDCMovieList::~KDCMovieList()
{

}

void KDCMovieList::setCurrentMovie( KDCMovie *movie )
{
	if ( movie != p_movie )
	{
		qDebug( "KDCMovieList::setCurrentMovie( %d )::begin", KDCLib::isNull( movie ) );
		clean();
		p_movie = movie;
		QTreeWidgetItem *item = new QTreeWidgetItem( this );
		item->setIcon( 0, KIcon("application-x-mplayer2") );
		item->setText( 0, movie->fileName() );
		setCurrentItem( item );
		qDebug( "KDCMovieList::setCurrentMovie()::end" );
	}
}

void KDCMovieList::setCurrentMovie( KDCMovieGroup *movie )
{
	if ( movie != p_group )
	{
		qDebug( "KDCMovieList::setCurrentMovie( %d )group::begin", KDCLib::isNull( movie ) );
		clean();
		p_group = movie;
		QTreeWidgetItem *item;
		for( int i=0; i<movie->count(); ++i )
		{
			item = new QTreeWidgetItem( this );
			item->setIcon( 0, KIcon("application-x-mplayer2") );
			item->setText( 0, movie->KDCMovieGroup::movie( i )->fileName() );
		}
		setCurrentItem( item );
		qDebug( "KDCMovieList::setCurrentMovie()group::end" );
	}
}

void KDCMovieList::clean()
{
	p_group = NULL;
	p_movie = NULL;
	clear();
}

bool KDCMovieList::dropMimeData( QTreeWidgetItem *parent, int index, const QMimeData *data, Qt::DropAction action )
{
	QList<QUrl> list = data->urls();
	int pos = list.first().toString().toInt();
	qDebug( "KDCMovieList::dropMimeData:: pos = %d", pos );
	if ( parent )
	{
		int newPos = indexOfTopLevelItem( parent );
		if ( newPos == pos )
			return FALSE;

		blockSignals( TRUE );
		QTreeWidgetItem *item = takeTopLevelItem( pos );
		insertTopLevelItem( newPos, item );
		p_group->move( pos, newPos );
		setCurrentItem( item );
		blockSignals( FALSE );
		return TRUE;
	}
	else
		return FALSE;
}

QStringList KDCMovieList::mimeTypes() const
{
	QStringList qstrList;
	qstrList.append("text/uri-list");
	return qstrList;
}

Qt::DropActions KDCMovieList::supportedDropActions() const
{
	return Qt::CopyAction;
}

void KDCMovieList::mouseMoveEvent( QMouseEvent *event )
{
	if ( !( event->buttons() & Qt::LeftButton ) )
		return;

	qDebug( "KDCMovieList::mouseMoveEvent()::currentItem = %d", KDCLib::isNull( currentItem() ) );
	if ( currentItem() )
	{
		QList<QUrl> list;
		list.append( QUrl( QString::number( indexOfTopLevelItem( currentItem() ) ) ) );
		QMimeData *mData = new QMimeData;
		QDrag *drag = new QDrag( this );
		mData->setUrls( list );
		drag->setMimeData( mData );

		qDebug( "KDCMovieList::mouseMoveEvent()::start draging" );
		drag->start( Qt::CopyAction );
	}
}

void KDCMovieList::on_currentItemChanged( QTreeWidgetItem *current, QTreeWidgetItem *previous )
{
	if ( current )
	{
		if ( p_group )
			emit currentMovieChanged( p_group->movie( indexOfTopLevelItem( current ) ) );
		else
			emit currentMovieChanged( p_movie );
	}
	else
		emit currentMovieChanged( NULL );
}

KDCMovie* KDCMovieList::currentMovie() const
{
	if ( p_movie )
		return p_movie;
	else
		return p_group->movie( indexOfTopLevelItem( currentItem() ) );
}
