/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcdiscoptionsstatusbar.h"

#include <QHBoxLayout>
#include <QPalette>

#include <KIcon>
#include <KLocale>

KDCDiscOptionsStatusBar::KDCDiscOptionsStatusBar( QWidget *parent )
	: QWidget( parent )
{
	p_mapper = new QSignalMapper( this );
	QHBoxLayout *hLayout = new QHBoxLayout( this );

	p_delLabel = new KDCLabel( "DEL" );
	p_delLabel->setToolTip( i18n("Delete oryginal files. For now it doesn't work.") );
	p_delAction = new KAction( KIcon("edit-delete"), i18n("Delete oryginal files"), this );
	p_delAction->setCheckable( TRUE );
	p_delAction->setData( QVariant( 0 ) );
	hLayout->addWidget( p_delLabel );
	p_mapper->setMapping( p_delLabel, 0 );
	connect( p_delLabel, SIGNAL( clicked() ), p_mapper, SLOT( map() ) );
	connect( p_delAction, SIGNAL( triggered( bool ) ), this, SLOT( on_triggered( bool ) ) );

	p_tempLabel = new KDCLabel( "TEMP" );
	p_tempLabel->setToolTip( i18n("Delete temporary files") );
	p_tempAction = new KAction( KIcon("edit-delete-shred"), i18n("Delete temporary files"), this );
	p_tempAction->setCheckable( TRUE );
	p_tempAction->setData( QVariant( 1 ) );
	hLayout->addWidget( p_tempLabel );
	p_mapper->setMapping( p_tempLabel, 1 );
	connect( p_tempLabel, SIGNAL( clicked() ), p_mapper, SLOT( map() ) );
	connect( p_tempAction, SIGNAL( triggered( bool ) ), this, SLOT( on_triggered( bool ) ) );

	p_strLabel = new KDCLabel( "STR" );
	p_strLabel->setToolTip( i18n("Create disc structure") );
	p_strAction = new KAction( i18n("Create disc structure"), this );
	p_strAction->setCheckable( TRUE );
	p_strAction->setData( QVariant( 2 ) );
	hLayout->addWidget( p_strLabel );
	p_mapper->setMapping( p_strLabel, 2 );
	connect( p_strLabel, SIGNAL( clicked() ), p_mapper, SLOT( map() ) );
	connect( p_strAction, SIGNAL( triggered( bool ) ), this, SLOT( on_triggered( bool ) ) );

	connect( p_mapper, SIGNAL( mapped( int ) ), this, SLOT( on_mapped( int ) ) );
	reset();
	setSignalsEnabled( FALSE );
}

KDCDiscOptionsStatusBar::~KDCDiscOptionsStatusBar()
{
	delete p_delLabel;
	delete p_delAction;
	delete p_strLabel;
	delete p_strAction;
	delete p_tempAction;
	delete p_tempLabel;
	delete p_mapper;
}

KAction* KDCDiscOptionsStatusBar::deleteFilesAction() const
{
	return p_delAction;
}

KAction* KDCDiscOptionsStatusBar::discStructureAction() const
{
	return p_strAction;
}

KAction* KDCDiscOptionsStatusBar::deleteTempFilesAction() const
{
	return p_tempAction;
}

void KDCDiscOptionsStatusBar::setDeleteFilesEnabled( bool enabled )
{
	p_delLabel->setData( QVariant( enabled ) );
	setAsChecked( p_delLabel, enabled );
	p_delAction->setChecked( enabled );
	emit deleteFilesStatusChanged( enabled );
}

void KDCDiscOptionsStatusBar::setStructureEnabled( bool enabled )
{
	p_strLabel->setData( QVariant( enabled ) );
	setAsChecked( p_strLabel, enabled );
	p_strAction->setChecked( enabled );
	emit structureStatusChanged( enabled );
}

void KDCDiscOptionsStatusBar::setDeleteTempFilesEnabled( bool enabled )
{
	p_tempLabel->setData( QVariant( enabled ) );
	setAsChecked( p_tempLabel, enabled );
	p_tempAction->setChecked( enabled );
	emit deleteTempFilesStatusChanged( enabled );
}

void KDCDiscOptionsStatusBar::on_mapped( int id )
{
	qDebug( "KDCDiscOptionsStatusBar::mapped( %d )", id );
	if ( id == 0 )
		setDeleteFilesEnabled( !p_delLabel->data().toBool() );
	else
		if ( id == 1 )
			setDeleteTempFilesEnabled( !p_tempLabel->data().toBool() );
	else
		setStructureEnabled( !p_strLabel->data().toBool() );
}

void KDCDiscOptionsStatusBar::on_triggered( bool enabled )
{
	KAction *action = qobject_cast<KAction*>( sender() );
	if ( action )
	{
		if ( action->data().toInt() == 0 )
			setDeleteFilesEnabled( enabled );
		else
			if ( action->data().toInt() == 1 )
				setDeleteTempFilesEnabled( enabled );
		else
			setStructureEnabled( enabled );
	}
}

void KDCDiscOptionsStatusBar::setAsChecked( KDCLabel *label, bool enabled )
{
	QPalette p;
	if ( enabled )
		p.setColor( QPalette::Active, QPalette::WindowText, p.color( QPalette::Active, QPalette::WindowText ) );
	else
		p.setColor( QPalette::Active, QPalette::WindowText, p.color( QPalette::Disabled, QPalette::WindowText ) );
	label->setPalette( p );
}

void KDCDiscOptionsStatusBar::reset()
{
	p_delLabel->setData( QVariant( FALSE ) );
	p_tempLabel->setData( QVariant( FALSE ) );
	p_strLabel->setData( QVariant( FALSE ) );
	setAsChecked( p_delLabel, FALSE );
	setAsChecked( p_tempLabel, FALSE );
	setAsChecked( p_strLabel, FALSE );
	p_delAction->setChecked( FALSE );
	p_tempAction->setChecked( FALSE );
	p_strAction->setChecked( FALSE );
}

void KDCDiscOptionsStatusBar::setSignalsEnabled( bool enabled )
{
	p_mapper->blockSignals( !enabled );
}
