/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcvideoqualitycombobox.h"

#include <QVariant>

#include <KLocale>

KDCVideoQualityComboBox::KDCVideoQualityComboBox(	const KDCMovie *movie, KDCVideoQualityComboBox::Mode mode,
																									QWidget *parent )
	: KComboBox( parent )
{
	if ( movie->videoStream()->mode() == KDC::Copy )
		return;

	p_mode = mode;
	if ( mode == MacroBlock )
	{
		if ( movie->videoStream()->hqDVDMode() == KDC::Disabled )
		{
			addItem( i18n("Use MBCMP (faster)"), KDC::MBCMP );
			addItem( i18n("Select the MB mode which needs the fewest bits"), KDC::MDB1 );
			addItem( i18n("Select the MB mode which has the best rate distortion"), KDC::MDB2 );
			setCurrentIndex( findData( movie->videoStream()->macroBlockMode() ) );
		}
	}
	else
	{
		addItem( i18n("Don't deinterlate"), KDC::None );
		addItem( i18n("Linear blend deinterlacing filter"), KDC::Linear );
		addItem( i18n("Median deinterlacing filter"), KDC::Median );
		addItem( i18n("FFmpeg deinterlacing filter ( default for RMVB )"), KDC::FFmpeg );
		setCurrentIndex( findData( movie->videoStream()->deinterlaceMode() ) );
	}

	connect( this, SIGNAL( currentIndexChanged( int ) ), this, SLOT( on_currentIndexChanged( int ) ) );
}

KDCVideoQualityComboBox::~KDCVideoQualityComboBox()
{

}

KDC::DeinterlaceMode KDCVideoQualityComboBox::currentDeinterlaceMode() const
{
	int data = itemData( currentIndex() ).toInt();
	if ( data == KDC::None )
		return KDC::None;
	else
		if ( data == KDC::Linear )
			return KDC::Linear;
	else
		if ( data == KDC::Median )
			return KDC::Median;
	else
		return KDC::FFmpeg;
}

KDC::MacroBlockMode KDCVideoQualityComboBox::currentMacroBlockMode() const
{
	int data = itemData( currentIndex() ).toInt();
	if ( data == KDC::MBCMP )
		return KDC::MBCMP;
	else
		if ( data == KDC::MDB1 )
			return KDC::MDB1;
	else
		return KDC::MDB2;
}

void KDCVideoQualityComboBox::on_currentIndexChanged( int index )
{
	if ( p_mode == MacroBlock )
		emit currentMacroBlockModeChanged( currentMacroBlockMode() );
	else
		emit currentDeinterlaceModeChanged( currentDeinterlaceMode() );
}
